package cn.minbb.hid.services;

import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.hardware.usb.UsbConstants;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbDeviceConnection;
import android.hardware.usb.UsbEndpoint;
import android.hardware.usb.UsbInterface;
import android.hardware.usb.UsbManager;
import android.os.IBinder;
import android.util.Log;
import android.widget.Toast;

import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import cn.minbb.hid.utils.ByteUtil;

public class DataService extends Service {

    private static final String TAG = "DataService";
    private UsbManager usbManager;
    private UsbDevice usbDevice;
    private UsbDeviceConnection usbDeviceConnection;
    private List<UsbInterface> usbInterfaceList = new ArrayList<>(8);
    private UsbEndpoint epBulkIn, epBulkOut, epControl, epIntEndpointIn, epIntEndpointOut;
    private int VendorID, ProductID;
    private Thread threadDataIn;
    private boolean isInterrupted = false;

    public DataService() {
    }

    @Override
    public IBinder onBind(Intent intent) {
        throw new UnsupportedOperationException("Not yet implemented");
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        super.onStartCommand(intent, flags, startId);
        Log.i(TAG, "数据接收服务启动");
        // 获取 USB 管理器
        usbManager = (UsbManager) getSystemService(Context.USB_SERVICE);

        // 枚举设备
        enumerateDevice(usbManager);
        // 查找设备接口
        getDeviceInterface();
        // 获取设备端点
        assignEndpoint(usbInterfaceList.get(0));
        // 打开连接通道
        openDevice(usbInterfaceList.get(0));
        return super.onStartCommand(intent, flags, startId);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (threadDataIn != null && !threadDataIn.isInterrupted()) {
            isInterrupted = true;
        }
        Log.i(TAG, "数据接收服务停止");
    }

    public static void start(Context context) {
        Intent starter = new Intent(context, DataService.class);
        context.startService(starter);
    }

    // 枚举设备
    private void enumerateDevice(UsbManager usbManager) {
        Log.i(TAG, "开始进行枚举设备");
        if (usbManager == null) {
            Log.i(TAG, "创建 UsbManager 失败，请重新启动应用");
        } else {
            HashMap<String, UsbDevice> deviceList = usbManager.getDeviceList();
            if (!(deviceList.isEmpty())) {
                // USB 设备列表不为空
                Log.i(TAG, "USB 设备列表不为空");
                for (UsbDevice device : deviceList.values()) {
                    // 输出设备信息
                    Log.i(TAG, "DeviceInfo: " + device.getVendorId() + " , " + device.getProductId());
                    // 保存设备 VID 和 PID
                    VendorID = device.getVendorId();
                    ProductID = device.getProductId();
                    // 保存匹配到的设备
                    if (VendorID == 1061 && ProductID == 33113) {
                        // 获取 USBDevice
                        this.usbDevice = device;
                        Log.i(TAG, "发现待匹配设备:" + device.getVendorId() + "," + device.getProductId());
                        Toast.makeText(getApplicationContext(), "发现待匹配设备", Toast.LENGTH_SHORT).show();
                    } else {
                        Log.i(TAG, "未发现指定设备");
                    }
                }
            } else {
                Log.e(TAG, "请连接USB设备至PAD！");
                Toast.makeText(getApplicationContext(), "请连接USB设备至PAD！", Toast.LENGTH_SHORT).show();
            }
        }
    }

    // 寻找设备接口
    private void getDeviceInterface() {
        if (usbDevice != null) {
            Log.d(TAG, "接口数量 = " + usbDevice.getInterfaceCount());
            for (int i = 0; i < usbDevice.getInterfaceCount(); i++) {
                UsbInterface inter = usbDevice.getInterface(i);
                // 保存设备接口
                usbInterfaceList.add(inter);
                Log.i(TAG, "成功获得设备接口 " + i + ": " + inter.getId());
            }
        } else {
            Log.e(TAG, "设备为空！");
        }
    }

    // 分配端点，IN | OUT，即输入输出；可以通过判断
    private UsbEndpoint assignEndpoint(UsbInterface usbInterface) {
        if (usbInterface == null) {
            Log.e(TAG, "USB 端点为空");
            return null;
        }
        int endpointCount = usbInterface.getEndpointCount();
        Log.i(TAG, "端点数量 = " + endpointCount);
        for (int i = 0; i < endpointCount; i++) {
            UsbEndpoint ep = usbInterface.getEndpoint(i);
            // look for bulk endpoint
            if (ep.getType() == UsbConstants.USB_ENDPOINT_XFER_BULK) {
                if (ep.getDirection() == UsbConstants.USB_DIR_OUT) {
                    epBulkOut = ep;
                    Log.i(TAG, "Find the BulkEndpointOut," + "index:" + i + ", 使用端点号：" + epBulkOut.getEndpointNumber());
                } else {
                    epBulkIn = ep;
                    Log.i(TAG, "Find the BulkEndpointIn:" + "index:" + i + ", 使用端点号：" + epBulkIn.getEndpointNumber());
                }
            }
            // look for contorl endpoint
            if (ep.getType() == UsbConstants.USB_ENDPOINT_XFER_CONTROL) {
                epControl = ep;
                Log.i(TAG, "Find the ControlEndPoint:" + "index:" + i + ", 使用端点号：" + epControl.getEndpointNumber());
            }
            // look for interrupte endpoint
            if (ep.getType() == UsbConstants.USB_ENDPOINT_XFER_INT) {
                if (ep.getDirection() == UsbConstants.USB_DIR_OUT) {
                    epIntEndpointOut = ep;
                    Log.i(TAG, "Find the InterruptEndpointOut:" + "index:" + i + ", 使用端点号：" + epIntEndpointOut.getEndpointNumber());
                }
                if (ep.getDirection() == UsbConstants.USB_DIR_IN) {
                    epIntEndpointIn = ep;
                    Log.i(TAG, "Find the InterruptEndpointIn:" + "index:" + i + ", 使用端点号：" + epIntEndpointIn.getEndpointNumber());
                }
            }
        }
        if (epBulkOut == null && epBulkIn == null && epControl == null && epIntEndpointOut == null && epIntEndpointIn == null) {
            throw new IllegalArgumentException("没有找到任何端点");
        }
        return epIntEndpointIn;
    }

    // 打开设备
    public void openDevice(UsbInterface usbInterface) {
        if (usbInterface != null) {
            UsbDeviceConnection conn = null;
            // 在 open 前判断是否有连接权限；对于连接权限可以静态分配，也可以动态分配权限
            if (usbManager.hasPermission(usbDevice)) {
                conn = usbManager.openDevice(usbDevice);
            } else {
                Log.e(TAG, "权限不足");
            }

            if (conn == null) {
                Log.e(TAG, "打开设备失败");
                return;
            }

            if (conn.claimInterface(usbInterface, true)) {
                usbDeviceConnection = conn;
                Log.i(TAG, "打开设备成功");
                Log.i(TAG, "Serial number = " + usbDeviceConnection.getSerial());
                Log.i(TAG, "FileDescriptor = " + usbDeviceConnection.getFileDescriptor());
                Log.i(TAG, "RawDescriptors = " + Arrays.toString(usbDeviceConnection.getRawDescriptors()));
                try {
                    Log.i(TAG, "RawDescriptors = " + new String(usbDeviceConnection.getRawDescriptors(), 0, 120, "utf-16le"));
                } catch (UnsupportedEncodingException e) {
                    e.printStackTrace();
                }

                // 开启线程接收数据
                threadDataIn = new Thread(() -> {
                    while (!isInterrupted) {
                        try {
                            byte[] buffer = new byte[255];
                            int transfer = usbDeviceConnection.bulkTransfer(epIntEndpointIn, buffer, buffer.length, 1000);
                            // 发送广播
                            Intent intent = new Intent();
                            if (transfer < 0) {
                                Log.e(TAG, "bulkIn返回输入为负数 = " + transfer);
                                intent.putExtra("success", false);
                                intent.putExtra("data", transfer);
                            } else {
                                Log.i(TAG, "数据接收成功 = " + transfer + " / " + Arrays.toString(buffer));
                                intent.putExtra("success", true);
                                intent.putExtra("data", buffer);
                            }
                            intent.setAction("cn.minbb.hid.services.DataService");
                            sendBroadcast(intent);
                        } catch (Exception e) {
                            e.printStackTrace();
                            break;
                        }
                    }
                });
                threadDataIn.start();
            } else {
                Log.e(TAG, "无法打开连接通道");
                conn.close();
            }
        }
    }

    // 从设备接收数据 bulkIn
    private byte[] receiveMessageFromPoint() {
        byte[] buffer = new byte[15];
        int transfer = usbDeviceConnection.bulkTransfer(epBulkIn, buffer, buffer.length, 2000);
        if (transfer < 0)
            Log.e(TAG, "bulkIn返回输入为负数 = " + transfer);
        else {
            Log.i(TAG, "数据接收成功 = " + transfer + " / " + Arrays.toString(buffer));
        }
        return buffer;
    }

    // 发送数据
    private void sendMessageToPoint(byte[] buffer) {
        // bulkOut传输
        int transfer = usbDeviceConnection.bulkTransfer(epBulkOut, buffer, buffer.length, 0);
        if (transfer < 0)
            Log.e(TAG, "bulkOut返回输出为负数 = " + transfer);
        else {
            Log.i(TAG, "数据发送成功 = " + transfer);
        }
    }
}
