package cn.minbb.hid.utils;

import android.content.Context;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbDeviceConnection;
import android.hardware.usb.UsbEndpoint;
import android.hardware.usb.UsbInterface;
import android.hardware.usb.UsbManager;
import android.util.Log;
import android.widget.Toast;

import java.util.HashMap;

import cn.minbb.hid.Commands;

public class USBHIDUtil {

    private Context context;

    private static final String TAG = "USBHIDUtil";
    private UsbManager usbManager;
    private UsbDevice usbDevice;
    private UsbInterface usbInterface;
    private UsbDeviceConnection usbDeviceConnection;
    private UsbEndpoint epIn, epOut;

    public USBHIDUtil(Context context) {
        this.context = context;
        usbManager = (UsbManager) context.getSystemService(Context.USB_SERVICE);
    }

    public void getAllHIDDevice() {
        if (usbManager == null) {
            Toast.makeText(context, "USB 管理器为空", Toast.LENGTH_SHORT).show();
            return;
        }
        HashMap<String, UsbDevice> deviceList = usbManager.getDeviceList();
        if (!deviceList.isEmpty()) {
            for (UsbDevice device : deviceList.values()) {
                int VendorID = 0x5302, ProductID = 0x636D;
                if (device.getVendorId() == VendorID && device.getProductId() == ProductID) {
                    usbDevice = device;
                    // 找到 Device 接口并分配相应端点
                    findInterfaceAndEndpoint();
                } else {
                    showToastAndLog("Not Find VID and PID");
                }
            }
        } else {
            showToastAndLog("未枚举到设备！请先连接设备，再重启程序。");
        }
    }

    // 查找接口和指定分配端点
    private void findInterfaceAndEndpoint() {
        if (usbDevice == null) {
            showToastAndLog("USB 设备不存在");
            return;
        }

        // 查找设备接口
        for (int i = 0; i < usbDevice.getInterfaceCount(); ) {
            // 获取设备接口，一般都是一个接口，可以打印getInterfaceCount()方法查看接口的个数，在这个接口上有两个端点，OUT 和 IN
            UsbInterface inter = usbDevice.getInterface(i);
            Log.d(TAG, i + " " + inter);
            if (inter.getInterfaceClass() == 8 && inter.getInterfaceSubclass() == 6 && inter.getInterfaceProtocol() == 80) {
                // HID设备的相关信息
                usbInterface = inter;
            }
            break;
        }

        if (usbInterface != null) {
            UsbDeviceConnection connection;
            // 判断是否有权限
            if (usbManager.hasPermission(usbDevice)) {
                // 打开设备，获取 UsbDeviceConnection 对象，连接设备，用于后面的通讯
                connection = usbManager.openDevice(usbDevice);
                if (connection == null) {
                    return;
                }
                if (connection.claimInterface(usbInterface, true)) {
                    usbDeviceConnection = connection;
                    getEndpoint(connection, usbInterface);
                } else {
                    connection.close();
                }
            } else {
                showToastAndLog("没有权限");
            }
        } else {
            showToastAndLog("USB 接口不存在");
        }
    }

    private void getEndpoint(UsbDeviceConnection connection, UsbInterface usbInterface) {
        if (usbInterface.getEndpoint(1) != null) {
            epOut = usbInterface.getEndpoint(1);
        }
        if (usbInterface.getEndpoint(0) != null) {
            epIn = usbInterface.getEndpoint(0);
        }
    }

    // 发送包操作，发送OUT + 发送COM + 接收IN
    private void sendPackage(byte[] command) {
        int ret = -100;
        int length = command.length;
        // 组织准备命令
        byte[] sendOut = Commands.OUT_S;
        sendOut[8] = (byte) (length & 0xff);
        sendOut[9] = (byte) ((length >> 8) & 0xff);
        sendOut[10] = (byte) ((length >> 16) & 0xff);
        sendOut[11] = (byte) ((length >> 24) & 0xff);
        // 1, 发送准备命令
        ret = usbDeviceConnection.bulkTransfer(epOut, sendOut, 31, 10000);
        if (ret != 31) {
            return;
        }
        // 2, 发送 COM
        ret = usbDeviceConnection.bulkTransfer(epOut, command, length, 10000);
        if (ret != length) {
            return;
        }
        // 3, 接收发送成功信息
        ret = usbDeviceConnection.bulkTransfer(epIn, Commands.IN, 13, 10000);
        if (ret != 13) {
            return;
        }
    }

    public void sendMessage() {
        byte[] sendOut = new byte[200];
        // sendOut[0] = 0x7349AB01;
        int transfer = usbDeviceConnection.bulkTransfer(epOut, sendOut, sendOut.length, 10000);
    }

    private void showToastAndLog(String message) {
        Log.d(TAG, message);
        Toast.makeText(context, message, Toast.LENGTH_SHORT).show();
    }
}
